"""Python environment module."""

import shlex
import subprocess
from dataclasses import dataclass


@dataclass
class PyEnv:
    """Python environment generic class."""

    name: str
    path: str

    def __eq__(self, item):
        return self.path == str(item)


class UserPyEnv(PyEnv):
    """User python environment."""

    scope = 'USER'


class TeamPyEnv(PyEnv):
    """Team python environment."""

    scope = 'TEAM'


class GlobalPyEnv(PyEnv):
    """Global python environment."""

    scope = 'GLOBAL'


def _micromamba_env_list(root_prefix='', base=False):
    """List micromamba environment list."""
    env_list = subprocess.run(
        shlex.split('/usr/local/bin/micromamba env list --quiet'),  # noqa: S603 (FIXME)
        env={'MAMBA_ROOT_PREFIX': root_prefix},
        capture_output=True,
        text=True,
        check=False,
    )

    return [map(str.strip, env.split()) for env in env_list.stdout.splitlines()[2 if base else 3 :]]


def get_pyenv_user(username: str) -> list:
    """List of python environment available to the user."""
    return [
        UserPyEnv(*env)
        for env in _micromamba_env_list(root_prefix=f'/micromamba/{username}/', base=True)
    ]


def get_pyenv_team(username: str) -> list:
    """List of python environment available to the user's team."""
    teams = []  # FIXME: pull user teams list from groups
    return [
        TeamPyEnv(*env)
        for team in teams
        for env in _micromamba_env_list(root_prefix=f'/micromamba/teams/{team}')
    ]


def get_pyenv_global() -> list:
    """List of python environment available globally."""
    return [GlobalPyEnv(*env) for env in _micromamba_env_list(root_prefix='/micromamba/operator/')]


def get_pyenv(username: str) -> list:
    """List of all the python environment available to a user."""
    return get_pyenv_user(username) + get_pyenv_team(username) + get_pyenv_global()
